-- helpers


function Bikinitools.Options:restrictCheats()
	Bikinitools.Options:disable(Bikinitools.Options.restrictedInMP);
end


function Bikinitools.Options:disableBrokenInMP()
	Bikinitools.Options:disable(Bikinitools.Options.brokenInMP);
end


function Bikinitools.Options:disableObsolete()
	Bikinitools.Options:disable(Bikinitools.Options.obsolete);
end


function Bikinitools.Options:disable(options)
	if type(options) == "table"
	then
		for _, option in ipairs(options)
		do
			Bikinitools.Options[option] = false;
		end
	else
		Bikinitools.Options[options] = false;
	end
end


function Bikinitools.Options:register(varName, defaultValue, needsRestart)
	if Bikinitools.Options[varName] == nil
	then
		Bikinitools.Options[varName] = defaultValue;
	end

	if needsRestart ~= nil and Bikinitools.Options.needRestart[varName] == nil
	then
		Bikinitools.Options.needRestart[varName] = true;
	end
end


function Bikinitools.Options:needsRestart(option)
    return Bikinitools:valueIsInArray(Bikinitools.Options.needRestart, option);
end


function Bikinitools.Options:isCheat(option)
	return Bikinitools:valueIsInArray(Bikinitools.Options.restrictedInMP, option);
end


function Bikinitools.Options:get(option, forceType)
    local value = Bikinitools.Options[option];

    if value ~= nil
    then
        if forceType == "number"
        then
            return tonumber(value);
        end

        return value;
    else
        return nil;
    end
end


function Bikinitools.Options:set(option, value, skipSave)
    Bikinitools.Options[option] = value;

    if skipSave ~= true
    then
        Bikinitools.Options:save();

        if option ~= "beReadyForFeatureToggles" and not Bikinitools.Options:get("beReadyForFeatureToggles") and Bikinitools.Options:needsRestart(option)
        then
            Bikinitools:message(getText("IGUI_bikinitools_O_ReturnToMainMenu"));
        end
    end
end


function Bikinitools.Options.toggle(option)
    Bikinitools.Options:set(option, not Bikinitools.Options:get(option));
end


function Bikinitools.Options:getSettingsFile()
	local settingsFile = Bikinitools.Config.settingsFile;

	if Bikinitools:gameIsMP() and not Bikinitools:playerIsAdmin()
	then
		settingsFile = Bikinitools.Config.restrictedSettingsFile;
	end

	return settingsFile;
end


function Bikinitools.Options:save()
    local file = getFileWriter(Bikinitools.Options:getSettingsFile(), true, false);

    for option, value in pairs(Bikinitools.Options)
    do
        if type(value) ~= "function" and type(value) ~= "table" and option ~= "fullyDisableBT"
        then
            file:write(option .. "=" .. tostring(value) .. "\n");
        end
    end

    file:close();
end


function Bikinitools.Options:load()
    local file = Bikinitools.Options:getSettingsFile();
	local handle = getFileReader(file, false);

    if handle
    then
		Bikinitools.Config.lastLoadedSettingsFile = file;

        while true
        do
            local line = handle:readLine();

            if line == nil --or line == ""
            then
                break;
            end

            local setting = {};

            for part in (line .. "="):gmatch("(.-)=")
            do
                if part == "true"
                then
                    part = true;
                elseif part == "false"
                then
                    part = false;
                end

                table.insert(setting, part);
            end

            if setting[1] ~= "fullyDisableBT"
            then
                Bikinitools.Options:set(setting[1], setting[2], true);
            end
        end

        handle:close();
    end
end


-- context menu helpers


function Bikinitools.Options:contextMenuItemsContainCheatItem(items)
	return Bikinitools.Options:contextMenuItemsContainedInList(items, Bikinitools.Options.restrictedInMP);
end


function Bikinitools.Options:contextMenuItemsContainMPDisabledItem(items)
	return Bikinitools.Options:contextMenuItemsContainedInList(items, Bikinitools.Options.brokenInMP);
end


function Bikinitools.Options:contextMenuItemsContainedInList(items, list)
	if not items
	then
		return false;
	end

	for i, option in ipairs(items)
	do
		if Bikinitools:valueIsInArray(list, option)
		then
			return true;
		end
	end

	return false;
end


function Bikinitools.Options:createContextMenuItem(configMenu, option, handler, params)
	local tKey = "IGUI_bikinitools_O_" .. Bikinitools:ucFirst(option);
    local title = getTextOrNull(tKey);
	local tip = "";
	local disabled = false;

	if not title
	then
		title = option;
	end

    if Bikinitools.Options:needsRestart(option)
    then
        title = title .. " [*]";
    end

	if option == "fullyDisableBT"
	then
		tip = getText(tKey .. "_Tooltip", Bikinitools.Config.restrictedSettingsFile);

		handler = function()
			Bikinitools:showModalDialog(getText("IGUI_bikinitools_O_DisabledBTBtn"), function()
				Bikinitools.Options:set(option, true);
			end);
		end
	else
		tip = getTextOrNull(tKey .. "_Tooltip");
	end

    if option == "enableIconOverride" and getActivatedMods():contains("CheatMenuRB")
    then
        disabled = true;
        tip = getText("IGUI_bikinitools_O_ModIncompatibility");
    end

	if not Bikinitools:playerCanCheat() and Bikinitools:valueIsInArray(Bikinitools.Options.restrictedInMP, option)
	then
		tip = getText("IGUI_bikinitools_O_DisabledInMP");
		disabled = true;
	end

    Bikinitools:addContextMenuItem(configMenu,
        title, handler, params, {
			disabled = disabled,
            icon = Bikinitools.Options[option] == true,
            tip = tip
        }
    );
end


function Bikinitools.Options:fillContextMenuFromStructure(targetMenu, structure)
	for title, options in pairs(structure)
    do
        if type(title) == "string"
        then
			--if (not Bikinitools:playerCanCheat() and Bikinitools.Options:contextMenuItemsContainCheatItem(options))
			--	or (Bikinitools:gameIsMP() and Bikinitools.Options:contextMenuItemsContainMPDisabledItem(options))
			--then
				--Bikinitools:addContextMenuItem(targetMenu,
					--getText(title), nil, nil, {
						--tip = getText("IGUI_bikinitools_O_DisabledInMP"),
						--disabled = true,
					--}
				--);
			--else
				local subMenu = Bikinitools:addContextMenuSubMenu(targetMenu,
					getTextOrNull(title) or title, nil, nil, {
						tickTexture = Bikinitools.Config.icons.checkmark2,
					}
				);

				if Bikinitools:countArrayElements(options) > 1 and title ~= "bikinitools"
				then
					Bikinitools.Options:createContextMenuItem(subMenu,
						"Context_EnableAll",
						function(options)
							for i, option in ipairs(options)
							do
								local value = true;

								if Bikinitools:gameIsMP() and not Bikinitools:playerCanCheat() and Bikinitools.Options:isCheat(option)
								then
									value = false;
								end

								Bikinitools.Options:set(option, value);
							end
						end, {
							options
						}
					);
				end

				for i, option in ipairs(options)
				do
					Bikinitools.Options:createContextMenuItem(subMenu,
						option, Bikinitools.Options.toggle, {
							option
						}
					);
				end
			--end
        else
            Bikinitools.Options:createContextMenuItem(targetMenu,
                options, Bikinitools.Options.toggle, {
                    options
                }
            );
        end
    end
end


function Bikinitools.Options.createConfigContextMenu(context)
    Bikinitools:addContextMenuHeader(context, getText("IGUI_bikinitools_O_Context_Configuration"));

    local configMenu = Bikinitools:addContextMenuSubMenu(context,
        getText("IGUI_bikinitools_O_Context_ToggleSettings") .. " [" .. Bikinitools:getDiagString() .. "]",
        nil, nil, {
            tickTexture = Bikinitools.Config.icons.checkmark2,
        }
    );

	Bikinitools.Options:fillContextMenuFromStructure(configMenu, Bikinitools.Options.configMenuItems);

	if getDebug() and Bikinitools:playerCanCheat()
	then
		local debugMenu = Bikinitools:addContextMenuSubMenu(configMenu, "[DANGER ZONE]");

		Bikinitools:addContextMenuHeader(debugMenu, "This is experimental stuff. Use at your own risk.")

		Bikinitools.Options:fillContextMenuFromStructure(debugMenu, {
			["Vehicle position saver"] = {
				"enableVehiclePositionSaver",
			},
		});
	end

    return configMenu;
end


-- load settings


Bikinitools.Options:load();
Bikinitools.Options:save();