-- search functions and filters


function Bikinitools.Grid:sortSquaresByDistanceToSquare(squares, refSquare, afterSortFn)
	local sorted = {};

	local squaresAndDistance = {};
	local refX = refSquare:getX();
	local refY = refSquare:getY();

	for i, square in ipairs(squares)
	do
		table.insert(squaresAndDistance, {
			d = refSquare:DistToProper(square),
			s = square,
			z = square:getZ(),
		});
	end

	table.sort(squaresAndDistance, function(a, b)
		return a.s:getZ() <= b.s:getZ() and a.d <= b.d;
	end);

	for i, item in ipairs(squaresAndDistance)
	do
		--Bikinitools:message(i .. " = " .. item.d);

		table.insert(sorted, item.s);
	end

	return sorted;
end


function Bikinitools.Grid:applyFilter(squares, filterBy, filterValue)
	if filterBy == "soilType"
    then
        squares = Bikinitools.Grid:filterSquaresBySoilType(squares, filterValue);
    elseif filterBy == "hasCorpse"
    then
        squares = Bikinitools.Grid:filterSquaresByCorpses(squares, filterValue);
    elseif filterBy == "hasTree"
    then
        squares = Bikinitools.Grid:filterSquaresByHasTree(squares);
    elseif filterBy == "grass"
    then
        squares = Bikinitools.Grid:filterSquaresByHasGrass(squares);
    elseif filterBy == "plants"
    then
        squares = Bikinitools.Grid:filterSquaresByHasPlants(squares);
    elseif filterBy == "grassOrPlants"
    then
        squares = Bikinitools.Grid:filterSquaresByHasGrassOrPlants(squares);
    elseif filterBy == "hasScrappable"
	then
		squares = Bikinitools.Grid:filterSquaresByHasScrappable(squares, filterValue);
	elseif filterBy == "hasFilledContainer"
	then
		squares = Bikinitools.Grid:filterSquaresByHasFilledContainer(squares);
	end

	return squares;
end


function Bikinitools.Grid:moveSquaresWithDoorsToBeginning(squares)
	local doors = {};

	for i = #squares, 1, -1
	do
		if squares[i]:haveDoor()
		then
			table.insert(doors, table.remove(squares, i));
		end
	end

	return Bikinitools:concatArrays(Bikinitools:reverseArray(doors), squares);
end


function Bikinitools.Grid:highlightSquares(squares, state)
    for i, square in ipairs(squares)
    do
        Bikinitools.Square:highlight(square, state);
    end
end


function Bikinitools.Grid:getAllSquaresIn3x3GridRadius(centerSquare, radius, filterBy, filterValue)
    local cell = getWorld():getCell();

    local rTimes3 = (3 * radius);
    local centerX = centerSquare:getX();
    local centerY = centerSquare:getY();
    local centerZ = centerSquare:getZ();
    local boundsStartX = centerX - rTimes3;
    local boundsStartY = centerY - rTimes3;
    local boundsEndX = centerX + rTimes3;
    local boundsEndY = centerY + rTimes3;

    local squares = {};

    for xOffset = 0, boundsEndX - boundsStartX
    do
        for yOffset = 0, boundsEndY - boundsStartY
        do
            table.insert(squares, cell:getGridSquare(boundsStartX + xOffset, boundsStartY + yOffset, centerZ));
        end
    end

	if filterBy
	then
		squares = Bikinitools.Grid:applyFilter(squares, filterBy, filterValue);
	end

    return squares;
end


function Bikinitools.Grid:getAllSquaresInRoom(room, filterBy, filterValue)
	local result = {};

	if room
	then
		local squares = room:getSquares();

		for i = 0, squares:size() - 1
		do
			table.insert(result, squares:get(i));
		end
	end

	if filterBy
	then
		result = Bikinitools.Grid:applyFilter(result, filterBy, filterValue);
	end

	return result;
end


function Bikinitools.Grid:getAllSquaresInBuilding(building, filterBy, filterValue)
	local result = {};

	if building and building:getRoomsNumber() > 0
	then
		local rooms = building:getDef():getRooms();

		for i = 0, rooms:size() - 1
		do
			local freeSquare = rooms:get(i):getFreeSquare();

			if freeSquare
			then
				squares = Bikinitools:concatArrays(result, Bikinitools.Grid:getAllSquaresInRoom(freeSquare:getRoom()));
			end
		end
	end

	if filterBy
	then
		result = Bikinitools.Grid:applyFilter(result, filterBy, filterValue);
	end

	return result;
end


function Bikinitools.Grid:filterSquaresBy(squares, filterFn)
    local filteredSquares = {};

    for i = #squares, 1, -1
    do
        local square = squares[i];

        if filterFn(square)
        then
            table.insert(filteredSquares, square);
        end
    end

    return filteredSquares;
end


function Bikinitools.Grid:filterSquaresByHasScrappable(squares, allScrappables)
	return Bikinitools.Grid:filterSquaresBy(
		squares,
		function(square)
			return Bikinitools.Square:getFirstScrappableObject(square, Bikinitools:getPlayer(), allScrappables) or false;
		end
	);
end


function Bikinitools.Grid:filterSquaresByHasFilledContainer(squares)
	return Bikinitools.Grid:filterSquaresBy(
		squares,
		function(square, player)
			return Bikinitools.Square:getFirstFilledContainer(square) or false;
		end,
		player
	);
end


function Bikinitools.Grid:filterSquaresBySoilType(squares, refSoilType)
    return Bikinitools.Grid:filterSquaresBy(squares, function(square)
        local soilType, soilTile = Bikinitools.Square:getSoilType(square);

        return soilType and soilType == refSoilType;
    end);
end


function Bikinitools.Grid:filterSquaresByCorpses(squares, hasCorpse)
    return Bikinitools.Grid:filterSquaresBy(squares, function(square)
        local corpses = square:getDeadBodys();

        if hasCorpse
        then
            return corpses:size() > 0;
        else
            return corpses:size() == 0;
        end
    end);
end


function Bikinitools.Grid:filterSquaresByHasTree(squares)
    return Bikinitools.Grid:filterSquaresBy(squares, function(square)
        return square:getTree() or false;
    end);
end


function Bikinitools.Grid:filterSquaresByHasGrass(squares)
    return Bikinitools.Grid:filterSquaresBy(squares, function(square)
        return Bikinitools.Square:getGrass(square) or false;
    end);
end


function Bikinitools.Grid:filterSquaresByHasPlants(squares)
    return Bikinitools.Grid:filterSquaresBy(squares, function(square)
        return Bikinitools.Square:getPlant(square) or false;
    end);
end


function Bikinitools.Grid:filterSquaresByHasGrassOrPlants(squares)
    return Bikinitools.Grid:filterSquaresBy(squares, function(square)
        return Bikinitools.Square:getGrass(square) or Bikinitools.Square:getPlant(square);
    end);
end
